#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaADXMAType {Butterworth_2, Butterworth_3, DEMA, Ehlers, EMA, Gauss_2, Gauss_3, Gauss_4, HoltEMA, LLMA, 
			SMA, SuperSmoother_2, SuperSmoother_3, T3, TEMA, TMA, VIDYA, VWMA, WMA, ZeroLagHATEMA, ZeroLagTEMA, ZLEMA}

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Directional Movement (anaDirectionalMovement). This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.
	/// </summary>
	[Description("This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.")]
	public class anaDirectionalMovement : Indicator
	{
		#region Variables
		private int					averagePeriod	= 27;
		private int					adxrPeriod		= 14;
		private int					volaPeriod		= 9;
		private int					period			= 0;
		private int					lookback		= 0;
		private double				phase			= 0.0;
		private double				amplification	= 0.7;
		private bool				wilderMode		= false;
		private T3CalcMode			calcMode		= T3CalcMode.FulksMatulich;
		private anaADXMAType		averageType		= anaADXMAType.EMA; 
		private int 				plot0Width 		= 2;
		private PlotStyle 			plot0Style 		= PlotStyle.Line;
		private DashStyle 			dash0Style 		= DashStyle.Solid;
		private int 				plot1Width 		= 2;
		private PlotStyle 			plot1Style 		= PlotStyle.Line;
		private DashStyle 			dash1Style 		= DashStyle.Solid;
		private int 				plot2Width 		= 2;
		private PlotStyle 			plot2Style 		= PlotStyle.Line;
		private DashStyle 			dash2Style 		= DashStyle.Solid;
		private int 				plot3Width 		= 2;
		private PlotStyle 			plot3Style 		= PlotStyle.Line;
		private DashStyle 			dash3Style 		= DashStyle.Dash;
		private int					upperlineValue	= 75;
		private int					lowerlineValue	= 25;
		private int					upperlineWidth	= 2;
		private DashStyle			upperlineStyle	= DashStyle.Dash;
		private int					lowerlineWidth	= 2;
		private DashStyle			lowerlineStyle	= DashStyle.Dash;
		private Color				adxRisingColor	= Color.MediumSeaGreen;
		private Color				adxFallingColor = Color.DarkGreen;
		private Color				diPlusColor		= Color.RoyalBlue;
		private Color				diMinusColor	= Color.Red;
		private Color				adxrColor		= Color.DarkOrange;
		private Color				upperlineColor	= Color.MediumOrchid;
		private Color				lowerlineColor	= Color.MediumOrchid;
		private DataSeries			dmPlus;
		private DataSeries			dmMinus;
		private DataSeries			tr;
		private DataSeries			dmi;
		private IDataSeries			avgDmPlus;
		private IDataSeries			avgDmMinus;
		private IDataSeries			avgTr;
		private IDataSeries			adx;
		private DataSeries			adxSeries;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Gray, 2), PlotStyle.Line, "ADX"));
			Add(new Plot(new Pen(Color.Gray, 2), PlotStyle.Line, "+DI"));
			Add(new Plot(new Pen(Color.Gray, 2), PlotStyle.Line, "-DI"));
			Add(new Plot(new Pen(Color.Gray, 2), PlotStyle.Line, "ADXR"));
			Add(new Line(Color.Gray, 75, "Upper"));
			Add(new Line(Color.Gray, 25, "Lower"));

			PlotsConfigurable = false;
			LinesConfigurable = false;
			
			dmPlus = new DataSeries(this);
			dmMinus = new DataSeries(this);
			tr = new DataSeries(this);
			dmi = new DataSeries(this);
			adxSeries = new DataSeries(this);
		}

		protected override void OnStartUp()
		{
			if(wilderMode)
				period =2*averagePeriod - 1;
			else
				period = averagePeriod;
			
			switch (averageType)
			{
				case anaADXMAType.Butterworth_2: 
					avgDmPlus = anaButterworthFilter(dmPlus, period, 2);	
					avgDmMinus = anaButterworthFilter(dmMinus, period, 2);	
					avgTr = anaButterworthFilter(tr, period, 2);	
					adx = anaButterworthFilter(dmi, period, 2);	
					break;
				case anaADXMAType.Butterworth_3: 
					avgDmPlus = anaButterworthFilter(dmPlus, period, 3);	
					avgDmMinus = anaButterworthFilter(dmMinus, period, 3);	
					avgTr = anaButterworthFilter(tr, period, 3);	
					adx = anaButterworthFilter(dmi, period, 3);	
					break;
				case anaADXMAType.DEMA: 
					avgDmPlus = DEMA(dmPlus, period);	
					avgDmMinus = DEMA(dmMinus, period);
					avgTr = DEMA(tr, period);
					adx = DEMA(dmi, period);
					break;
				case anaADXMAType.Ehlers: 
					avgDmPlus = anaEhlersFilter(dmPlus, period);	
					avgDmMinus = anaEhlersFilter(dmMinus, period);
					avgTr = anaEhlersFilter(tr, period);
					adx = anaEhlersFilter(dmi, period);
					break;
				case anaADXMAType.EMA:
					avgDmPlus = EMA(dmPlus, period);	
					avgDmMinus = EMA(dmMinus, period);
					avgTr = EMA(tr, period);
					adx = EMA(dmi, period);
					break;
				case anaADXMAType.Gauss_2: 
					avgDmPlus = anaGaussianFilter(dmPlus, period, 2);	
					avgDmMinus = anaGaussianFilter(dmMinus, period, 2);	
					avgTr = anaGaussianFilter(tr, period, 2);	
					adx = anaGaussianFilter(dmi, period, 2);	
					break;
				case anaADXMAType.Gauss_3: 
					avgDmPlus = anaGaussianFilter(dmPlus, period, 3);	
					avgDmMinus = anaGaussianFilter(dmMinus, period, 3);	
					avgTr = anaGaussianFilter(tr, period, 3);	
					adx = anaGaussianFilter(dmi, period, 3);	
					break;
				case anaADXMAType.Gauss_4: 
					avgDmPlus = anaGaussianFilter(dmPlus, period, 4);	
					avgDmMinus = anaGaussianFilter(dmMinus, period, 4);	
					avgTr = anaGaussianFilter(tr, period, 4);	
					adx = anaGaussianFilter(dmi, period, 4);	
					break;
				case anaADXMAType.HoltEMA: 
					avgDmPlus = HoltEMA(dmPlus, period, 2*period);	
					avgDmMinus = HoltEMA(dmMinus, period, 2*period);	
					avgTr = HoltEMA(tr, period, 2*period);	
					adx = HoltEMA(dmi, period, 2*period);	
					break;
				case anaADXMAType.LLMA: 
					avgDmPlus = LLMA(dmPlus, period, phase);	
					avgDmMinus = LLMA(dmMinus, period, phase);
					avgTr = LLMA(tr, period, phase);
					adx = LLMA(dmi, period, phase);
					break;
				case anaADXMAType.SMA: 
					avgDmPlus = SMA(dmPlus, period);	
					avgDmMinus = SMA(dmMinus, period);
					avgTr = SMA(tr, period);
					adx = SMA(dmi, period);
					break;
				case anaADXMAType.SuperSmoother_2: 
					avgDmPlus = anaSuperSmootherFilter(dmPlus, period, 2);	
					avgDmMinus = anaSuperSmootherFilter(dmMinus, period, 2);	
					avgTr = anaSuperSmootherFilter(tr, period, 2);	
					adx = anaSuperSmootherFilter(dmi, period, 2);	
					break;
				case anaADXMAType.SuperSmoother_3: 
					avgDmPlus = anaSuperSmootherFilter(dmPlus, period, 3);	
					avgDmMinus = anaSuperSmootherFilter(dmMinus, period, 3);	
					avgTr = anaSuperSmootherFilter(tr, period, 3);	
					adx = anaSuperSmootherFilter(dmi, period, 3);	
					break;
				case anaADXMAType.T3: 
					avgDmPlus = T3FulksMatulich(dmPlus, amplification, period, calcMode);	
					avgDmMinus = T3FulksMatulich(dmMinus, amplification, period, calcMode);	
					avgTr = T3FulksMatulich(tr, amplification, period, calcMode);	
					adx = T3FulksMatulich(dmi, amplification, period, calcMode);	
					break;
				case anaADXMAType.TEMA: 
					avgDmPlus = TEMA(dmPlus, period);	
					avgDmMinus = TEMA(dmMinus, period);
					avgTr = TEMA(tr, period);
					adx = TEMA(dmi, period);
					break;
				case anaADXMAType.TMA: 
					avgDmPlus = TMA(dmPlus, period);	
					avgDmMinus = TMA(dmMinus, period);
					avgTr = TMA(tr, period);
					adx = TMA(dmi, period);
					break;
				case anaADXMAType.VWMA: 
					avgDmPlus = VWMA(dmPlus, period);	
					avgDmMinus = VWMA(dmMinus, period);
					avgTr = VWMA(tr, period);
					adx = VWMA(dmi, period);
					break;
				case anaADXMAType.VIDYA: 
					avgDmPlus = VMA(dmPlus, period, volaPeriod);	
					avgDmMinus = VMA(dmMinus, period, volaPeriod);
					avgTr = VMA(tr, period, volaPeriod);
					adx = VMA(dmi, period, volaPeriod);
					break;
				case anaADXMAType.WMA: 
					avgDmPlus = WMA(dmPlus, period);	
					avgDmMinus = WMA(dmMinus, period);
					avgTr = WMA(tr, period);
					adx = WMA(dmi, period);
					break;
				case anaADXMAType.ZeroLagHATEMA: 
					avgDmPlus = anaZeroLagHATEMA(dmPlus, period);	
					avgDmMinus = anaZeroLagHATEMA(dmMinus, period);
					avgTr = anaZeroLagHATEMA(tr, period);
					adx = anaZeroLagHATEMA(dmi, period);
					break;
				case anaADXMAType.ZeroLagTEMA: 
					avgDmPlus = ZeroLagTEMA(dmPlus, period);	
					avgDmMinus = ZeroLagTEMA(dmMinus, period);
					avgTr = ZeroLagTEMA(tr, period);
					adx = ZeroLagTEMA(dmi, period);
					break;
				case anaADXMAType.ZLEMA: 
					avgDmPlus = ZLEMA(dmPlus, period);	
					avgDmMinus = ZLEMA(dmMinus, period);
					avgTr = ZLEMA(tr, period);
					adx = ZLEMA(dmi, period);
					break;
			}
			Plots[0].Pen.Width = plot0Width;
			Plots[0].PlotStyle = plot0Style;
			Plots[0].Pen.DashStyle = dash0Style;
			Plots[1].Pen.Width= plot1Width;
			Plots[1].PlotStyle = plot1Style;
			Plots[1].Pen.DashStyle = dash1Style;
			Plots[2].Pen.Width = plot2Width;
			Plots[2].PlotStyle = plot2Style;
			Plots[2].Pen.DashStyle = dash2Style;
			Plots[3].Pen.Width = plot3Width;
			Plots[3].PlotStyle = plot3Style;
			Plots[3].Pen.DashStyle = dash3Style;
			Plots[1].Pen.Color = diPlusColor;
			Plots[2].Pen.Color = diMinusColor;
			Plots[3].Pen.Color = adxrColor;
			Lines[0].Value = upperlineValue;
			Lines[0].Pen.Width = upperlineWidth;
			Lines[0].Pen.DashStyle = upperlineStyle;
			Lines[0].Pen.Color = upperlineColor;
			Lines[1].Value = lowerlineValue;
			Lines[1].Pen.Width = lowerlineWidth;
			Lines[1].Pen.DashStyle = lowerlineStyle;
			Lines[1].Pen.Color = lowerlineColor;
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			double trueRange = High[0] - Low[0];
			if (CurrentBar == 0)
			{
				tr.Set(trueRange);
				dmPlus.Set(0.0);
				dmMinus.Set(0.0);
				dmi.Set(50.0);
				ADX.Set(50.0);
				DiPlus.Set(0.0);
				DiMinus.Set(0.0);
				ADXR.Set(0.0);
				PlotColors[0][-Displacement] = adxRisingColor;
			}
			else
			{
				if(FirstTickOfBar)
					lookback = Math.Min(CurrentBar, adxrPeriod); 	
				
				tr.Set(Math.Max(Math.Abs(Low[0] - Close[1]), Math.Max(trueRange, Math.Abs(High[0] - Close[1]))));
				dmPlus.Set(High[0] - High[1] > Low[1] - Low[0] ? Math.Max(High[0] - High[1], 0) : 0);
				dmMinus.Set(Low[1] - Low[0] > High[0] - High[1] ? Math.Max(Low[1] - Low[0], 0) : 0);

				double diPlus	= 100 * (avgTr[0] == 0 ? 0 : avgDmPlus[0] / avgTr[0]);
				double diMinus	= 100 * (avgTr[0] == 0 ? 0 : avgDmMinus[0] / avgTr[0]);
				double diff		= Math.Abs(diPlus - diMinus);
				double sum		= diPlus + diMinus;

				dmi.Set(sum == 0 ? 50 : 100 * diff/sum);
				DiPlus.Set(Math.Max(0, Math.Min(100,diPlus)));
				DiMinus.Set(Math.Max(0, Math.Min(100,diMinus)));
				ADX.Set(Math.Max(0, Math.Min(100,adx[0])));
				ADXR.Set((ADX[0] + ADX[lookback])/2.0);
				
				if(ADX[0] > ADXR[0])
					PlotColors[0][-Displacement] = adxRisingColor;
				else if(ADX[0] < ADXR[0])
					PlotColors[0][-Displacement] = adxFallingColor;
				else
					PlotColors[0][-Displacement] = PlotColors[0][1-Displacement];
			}
		}

		#region Properties
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries ADX
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries DiPlus
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries DiMinus
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries ADXR
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculation of moving averages")]
		[GridCategory("Parameters")]
        [Gui.Design.DisplayNameAttribute("Average period")]
		public int AveragePeriod
		{
			get { return averagePeriod; }
			set { averagePeriod = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculation of signal line")]
		[GridCategory("Parameters")]
        [Gui.Design.DisplayNameAttribute("ADXR period")]
		public int ADXRPeriod
		{
			get { return adxrPeriod; }
			set { adxrPeriod = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[GridCategory("Parameters")]
        [Gui.Design.DisplayNameAttribute("LLMA phase")]
		public double Phase
		{
            get { return phase; }
            set { phase = Math.Min(Math.Max(-100, value), 100); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[GridCategory("Parameters")]
        [Gui.Design.DisplayNameAttribute("T3 amplification")]
		public double Amplification
		{
			get { return amplification; }
			set { amplification = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Period used to calculate the volatility index for Tushar Chande's VIDYA.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("VIDYA volatility period")]
		public int VolaPeriod
		{
			get { return volaPeriod; }
			set	{ volaPeriod = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[GridCategory("Options")]
        [Gui.Design.DisplayNameAttribute("Use Wilder's period")]
		public bool WilderMode
		{
			get { return wilderMode; }
			set { wilderMode = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Moving average type used for smoothing the directional movement")]
		[GridCategory("Options")]
		[Gui.Design.DisplayNameAttribute("Average type")]
		public anaADXMAType AverageType
		{
			get { return averageType; }
			set { averageType = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("ADX, ADX > ADXR")]
		public Color ADXRisingColor
		{
			get { return adxRisingColor; }
			set { adxRisingColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ADXRisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(adxRisingColor); }
			set { adxRisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("ADX, ADX < ADXR")]
		public Color ADXFallingColor
		{
			get { return adxFallingColor; }
			set { adxFallingColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ADXFallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(adxFallingColor); }
			set { adxFallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("+DI")]
		public Color DIPlusColor
		{
			get { return diPlusColor; }
			set { diPlusColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DIPlusColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(diPlusColor); }
			set { diPlusColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("-DI")]
		public Color DIMinusColor
		{
			get { return diMinusColor; }
			set { diMinusColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DIMinusColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(diMinusColor); }
			set { diMinusColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("ADXR")]
		public Color ADXRColor
		{
			get { return adxrColor; }
			set { adxrColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ADXRColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(adxrColor); }
			set { adxrColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Line Colors")]
		[Gui.Design.DisplayName("Upper line")]
		public Color UpperlineColor
		{
			get { return upperlineColor; }
			set { upperlineColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string UpperlineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upperlineColor); }
			set { upperlineColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Line Colors")]
		[Gui.Design.DisplayName("Lower line")]
		public Color LowerlineColor
		{
			get { return lowerlineColor; }
			set { lowerlineColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string LowerlineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(lowerlineColor); }
			set { lowerlineColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// </summary>
		[Description("Width for ADX.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line width ADX")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for ADX.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style ADX")]
		public PlotStyle Plot0Style
		{
			get { return plot0Style; }
			set { plot0Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for ADX.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash Style ADX")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		} 
		
		/// <summary>
		/// </summary>
		[Description("Width for +DI.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line width +DI")]
		public int Plot1Width
		{
			get { return plot1Width; }
			set { plot1Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for +DI.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style +DI")]
		public PlotStyle Plot1Style
		{
			get { return plot1Style; }
			set { plot1Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for +DI.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style +DI")]
		public DashStyle Dash1Style
		{
			get { return dash1Style; }
			set { dash1Style = value; }
		} 

		/// <summary>
		/// </summary>
		[Description("Width for -DI.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line width -DI")]
		public int Plot2Width
		{
			get { return plot2Width; }
			set { plot2Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for -DI.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style -DI")]
		public PlotStyle Plot2Style
		{
			get { return plot2Style; }
			set { plot2Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for -DI.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style -DI")]
		public DashStyle Dash2Style
		{
			get { return dash2Style; }
			set { dash2Style = value; }
		} 
		
		/// </summary>
		[Description("Width for ADXR.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line width ADXR")]
		public int Plot3Width
		{
			get { return plot3Width; }
			set { plot3Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for ADXR.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style ADXR")]
		public PlotStyle Plot3Style
		{
			get { return plot3Style; }
			set { plot3Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for ADXR.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash Style ADXR")]
		public DashStyle Dash3Style
		{
			get { return dash3Style; }
			set { dash3Style = value; }
		} 

		/// </summary>
		[Description("Value for upper line.")]
		[Category("Line Parameters")]
		[Gui.Design.DisplayNameAttribute("Value upper line")]
		public int UpperlineValue
		{
			get { return upperlineValue; }
			set { upperlineValue = Math.Min(100, Math.Max(0, value)); }
		}
		
		/// </summary>
		[Description("Value for lower line.")]
		[Category("Line Parameters")]
		[Gui.Design.DisplayNameAttribute("Value lower line")]
		public int LowerlineValue
		{
			get { return lowerlineValue; }
			set { lowerlineValue = Math.Min(100, Math.Max(0, value)); }
		}
		
		/// </summary>
		[Description("Width for upper line.")]
		[Category("Line Parameters")]
		[Gui.Design.DisplayNameAttribute("Width upper line")]
		public int UpperlineWidth
		{
			get { return upperlineWidth; }
			set { upperlineWidth = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for upper line.")]
		[Category("Line Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style upper line")]
		public DashStyle UpperlineStyle
		{
			get { return upperlineStyle; }
			set { upperlineStyle = value; }
		} 		
		
		/// </summary>
		[Description("Width for lower line.")]
		[Category("Line Parameters")]
		[Gui.Design.DisplayNameAttribute("Width lower line")]
		public int LowerlineWidth
		{
			get { return lowerlineWidth; }
			set { lowerlineWidth = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for lower line.")]
		[Category("Line Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style lower line")]
		public DashStyle LowerlineStyle
		{
			get { return lowerlineStyle; }
			set { lowerlineStyle = value; }
		} 		
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaDirectionalMovement[] cacheanaDirectionalMovement = null;

        private static anaDirectionalMovement checkanaDirectionalMovement = new anaDirectionalMovement();

        /// <summary>
        /// This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.
        /// </summary>
        /// <returns></returns>
        public anaDirectionalMovement anaDirectionalMovement(int aDXRPeriod, double amplification, int averagePeriod, anaADXMAType averageType, double phase, int volaPeriod, bool wilderMode)
        {
            return anaDirectionalMovement(Input, aDXRPeriod, amplification, averagePeriod, averageType, phase, volaPeriod, wilderMode);
        }

        /// <summary>
        /// This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.
        /// </summary>
        /// <returns></returns>
        public anaDirectionalMovement anaDirectionalMovement(Data.IDataSeries input, int aDXRPeriod, double amplification, int averagePeriod, anaADXMAType averageType, double phase, int volaPeriod, bool wilderMode)
        {
            if (cacheanaDirectionalMovement != null)
                for (int idx = 0; idx < cacheanaDirectionalMovement.Length; idx++)
                    if (cacheanaDirectionalMovement[idx].ADXRPeriod == aDXRPeriod && Math.Abs(cacheanaDirectionalMovement[idx].Amplification - amplification) <= double.Epsilon && cacheanaDirectionalMovement[idx].AveragePeriod == averagePeriod && cacheanaDirectionalMovement[idx].AverageType == averageType && Math.Abs(cacheanaDirectionalMovement[idx].Phase - phase) <= double.Epsilon && cacheanaDirectionalMovement[idx].VolaPeriod == volaPeriod && cacheanaDirectionalMovement[idx].WilderMode == wilderMode && cacheanaDirectionalMovement[idx].EqualsInput(input))
                        return cacheanaDirectionalMovement[idx];

            lock (checkanaDirectionalMovement)
            {
                checkanaDirectionalMovement.ADXRPeriod = aDXRPeriod;
                aDXRPeriod = checkanaDirectionalMovement.ADXRPeriod;
                checkanaDirectionalMovement.Amplification = amplification;
                amplification = checkanaDirectionalMovement.Amplification;
                checkanaDirectionalMovement.AveragePeriod = averagePeriod;
                averagePeriod = checkanaDirectionalMovement.AveragePeriod;
                checkanaDirectionalMovement.AverageType = averageType;
                averageType = checkanaDirectionalMovement.AverageType;
                checkanaDirectionalMovement.Phase = phase;
                phase = checkanaDirectionalMovement.Phase;
                checkanaDirectionalMovement.VolaPeriod = volaPeriod;
                volaPeriod = checkanaDirectionalMovement.VolaPeriod;
                checkanaDirectionalMovement.WilderMode = wilderMode;
                wilderMode = checkanaDirectionalMovement.WilderMode;

                if (cacheanaDirectionalMovement != null)
                    for (int idx = 0; idx < cacheanaDirectionalMovement.Length; idx++)
                        if (cacheanaDirectionalMovement[idx].ADXRPeriod == aDXRPeriod && Math.Abs(cacheanaDirectionalMovement[idx].Amplification - amplification) <= double.Epsilon && cacheanaDirectionalMovement[idx].AveragePeriod == averagePeriod && cacheanaDirectionalMovement[idx].AverageType == averageType && Math.Abs(cacheanaDirectionalMovement[idx].Phase - phase) <= double.Epsilon && cacheanaDirectionalMovement[idx].VolaPeriod == volaPeriod && cacheanaDirectionalMovement[idx].WilderMode == wilderMode && cacheanaDirectionalMovement[idx].EqualsInput(input))
                            return cacheanaDirectionalMovement[idx];

                anaDirectionalMovement indicator = new anaDirectionalMovement();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ADXRPeriod = aDXRPeriod;
                indicator.Amplification = amplification;
                indicator.AveragePeriod = averagePeriod;
                indicator.AverageType = averageType;
                indicator.Phase = phase;
                indicator.VolaPeriod = volaPeriod;
                indicator.WilderMode = wilderMode;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaDirectionalMovement[] tmp = new anaDirectionalMovement[cacheanaDirectionalMovement == null ? 1 : cacheanaDirectionalMovement.Length + 1];
                if (cacheanaDirectionalMovement != null)
                    cacheanaDirectionalMovement.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaDirectionalMovement = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaDirectionalMovement anaDirectionalMovement(int aDXRPeriod, double amplification, int averagePeriod, anaADXMAType averageType, double phase, int volaPeriod, bool wilderMode)
        {
            return _indicator.anaDirectionalMovement(Input, aDXRPeriod, amplification, averagePeriod, averageType, phase, volaPeriod, wilderMode);
        }

        /// <summary>
        /// This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaDirectionalMovement anaDirectionalMovement(Data.IDataSeries input, int aDXRPeriod, double amplification, int averagePeriod, anaADXMAType averageType, double phase, int volaPeriod, bool wilderMode)
        {
            return _indicator.anaDirectionalMovement(input, aDXRPeriod, amplification, averagePeriod, averageType, phase, volaPeriod, wilderMode);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaDirectionalMovement anaDirectionalMovement(int aDXRPeriod, double amplification, int averagePeriod, anaADXMAType averageType, double phase, int volaPeriod, bool wilderMode)
        {
            return _indicator.anaDirectionalMovement(Input, aDXRPeriod, amplification, averagePeriod, averageType, phase, volaPeriod, wilderMode);
        }

        /// <summary>
        /// This is the same indicator as the ADX, with the addition of the two directional movement indicators +DI and -DI. +DI and -DI measure upward and downward momentum. A buy signal is generated when +DI crosses -DI to the upside. A sell signal is generated when -DI crosses +DI to the downside.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaDirectionalMovement anaDirectionalMovement(Data.IDataSeries input, int aDXRPeriod, double amplification, int averagePeriod, anaADXMAType averageType, double phase, int volaPeriod, bool wilderMode)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaDirectionalMovement(input, aDXRPeriod, amplification, averagePeriod, averageType, phase, volaPeriod, wilderMode);
        }
    }
}
#endregion
